package solutions.reslate.entertainment.spigotresourcesync;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Collection;
import java.util.HashSet;
import java.util.logging.Logger;

import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.FileConfiguration;

import solutions.reslate.entertainment.spigotresourcesync.data.Configuration;
import solutions.reslate.entertainment.spigotresourcesync.events.ObjectLoadListener;
import solutions.reslate.entertainment.spigotresourcesync.serialisation.Serialiser;

public class ConfigManager extends FileConfiguration {
    private Serialiser<Configuration> serialiser;
    private Configuration config;
    private File file;
    private Logger logger;
    private Collection<ObjectLoadListener<Configuration>> configLoadListener;


    public ConfigManager(File file, Serialiser<Configuration> serialiser, Logger logger) {
        super();
        this.serialiser = serialiser;
        this.config = new Configuration();
        this.file = file;
        this.logger = logger;
        this.configLoadListener = new HashSet<>();
    }

    public void flush() {
        try {
            save(this.file);
        } catch (IOException e) {
            logger.severe(String.format("Unable to save configuration to \"%s\". %s", this.file.getAbsolutePath(), e.getMessage()));
        }
    }

    @Override
    public void save(String file) throws IOException {
        this.save(new File(file));
    }

    @Override
    public void save(File file) throws IOException {
        this.serialiser.flushTo(file, this.config);
        logger.info(String.format("Saved configuration to \"%s\".", this.file.getAbsolutePath()));
    }

    public void load() {
        try {
            this.load(file);
        } catch (IOException | InvalidConfigurationException e) {
            logger.severe(String.format("Unable to load configuration from \"%s\".", file.getAbsolutePath(), e.getMessage()));
            if (file.exists()) {
                logger.severe(String.format("Found pre-existing file. No overwriting will occur. Delete \"%s\" and restart to generate new configuration.", this.file.getAbsolutePath()));
            } else {
                logger.info(String.format("Generating new configuration file at \"%s\".", this.file.getPath()));
                if (!this.file.getParentFile().exists()) {
                    this.file.getParentFile().mkdirs();
                }
                flush();
            }
        }
    }


    @Override
    public void load(String file) throws FileNotFoundException, IOException, InvalidConfigurationException {
        this.load(new File(file));
    }
    
    @Override
    public void load(File file) throws FileNotFoundException, IOException, InvalidConfigurationException {
        try (FileReader reader = new FileReader(file)) {
            this.load(reader);
        }
    }

    @Override
    public void load(Reader reader) throws IOException, InvalidConfigurationException {
        this.config = this.serialiser.load(reader, Configuration.class);
        logger.info(String.format("Successfully loaded configuration from \"%s\".", this.file.getAbsolutePath()));
        onLoadConfig();
    }

    public Configuration getConfiguration() {
        return config;
    }

    private void onLoadConfig() {
        for (ObjectLoadListener<Configuration> objectUpdateListener : configLoadListener) {
            objectUpdateListener.objectLoaded(config);
        }
    }

    @Override
    public String saveToString() {
        return serialiser.serialize(this.config);
    }

    @Override
    public void loadFromString(String contents) throws InvalidConfigurationException {
        this.config = serialiser.deserialize(contents, Configuration.class);
        onLoadConfig();
    }

    public void addConfigLoadListener(ObjectLoadListener<Configuration> listener) {
        this.configLoadListener.add(listener);
    }

    public void removeConfigLoadListener(ObjectLoadListener<Configuration> listener) {
        this.configLoadListener.remove(listener);
    }

    public void resetConfiguration() {
        this.config = new Configuration();
    }

}
